package cn.com.qmth.examcloud.web.interceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.http.HttpStatus;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;

import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.logging.ExamCloudLog;
import cn.com.qmth.examcloud.commons.logging.ExamCloudLogFactory;
import cn.com.qmth.examcloud.commons.util.ThreadLocalUtil;
import cn.com.qmth.examcloud.web.redis.RedisClient;
import cn.com.qmth.examcloud.web.support.ServletUtil;
import cn.com.qmth.examcloud.web.support.StatusResponse;

/**
 * 顺序锁拦截器
 *
 * @author WANGWEI
 * @date 2018年11月12日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
public class SeqlockInterceptor implements HandlerInterceptor {

	private static final ExamCloudLog LOG = ExamCloudLogFactory.getLog(SeqlockInterceptor.class);

	/**
	 * redis client
	 */
	private RedisClient redisClient;

	private static final String LOCK_PREFIX = "$_lock:";

	/**
	 * 构造函数
	 *
	 * @param redisClient
	 * @param exclusions
	 */
	public SeqlockInterceptor(RedisClient redisClient) {
		super();
		this.redisClient = redisClient;
	}

	@Override
	public boolean preHandle(HttpServletRequest request, HttpServletResponse response,
			Object handler) throws Exception {
		LOG.debug("preHandle... ...");

		User user = null;
		Object userAttribute = request.getAttribute("$accessUser");
		if (null == userAttribute) {
			return true;
		} else {
			user = (User) userAttribute;
		}

		if (handler instanceof HandlerMethod) {
			HandlerMethod handlerMethod = (HandlerMethod) handler;
			Seqlock seqlock = handlerMethod.getMethodAnnotation(Seqlock.class);
			SessionSeqlock sessionSeqlock = handlerMethod.getMethodAnnotation(SessionSeqlock.class);
			if (null != seqlock) {
				String mappingPath = (String) request.getAttribute("$mappingPath");
				String key = LOCK_PREFIX + mappingPath;

				if (redisClient.setIfAbsent(key, ThreadLocalUtil.getTraceID(), 60 * 5)) {
					if (LOG.isDebugEnabled()) {
						LOG.debug("global locked");
					}
					return true;
				} else {
					response.setStatus(HttpStatus.CONFLICT.value());
					ServletUtil.returnJson(new StatusResponse("409", "请稍后重试... ..."), response);
					return false;
				}

			} else if (null != sessionSeqlock) {
				String mappingPath = (String) request.getAttribute("$mappingPath");
				String key = LOCK_PREFIX + user.getUserId() + ":" + mappingPath;

				if (redisClient.setIfAbsent(key, ThreadLocalUtil.getTraceID(), 60 * 5)) {
					if (LOG.isDebugEnabled()) {
						LOG.debug("sesssion locked");
					}
					return true;
				} else {
					response.setStatus(HttpStatus.CONFLICT.value());
					ServletUtil.returnJson(new StatusResponse("409", "请稍后重试... ..."), response);
					return false;
				}
			}
		}

		return true;
	}

	@Override
	public void postHandle(HttpServletRequest request, HttpServletResponse response, Object handler,
			ModelAndView modelAndView) throws Exception {
		LOG.debug("postHandle... ...");
	}

	@Override
	public void afterCompletion(HttpServletRequest request, HttpServletResponse response,
			Object handler, Exception ex) throws Exception {
		LOG.debug("afterCompletion... ...");

		String mappingPath = (String) request.getAttribute("$mappingPath");
		String key = LOCK_PREFIX + mappingPath;
		redisClient.delete(key);

		User user = null;
		Object userAttribute = request.getAttribute("$accessUser");
		if (null != userAttribute) {
			user = (User) userAttribute;
			String sessionKey = LOCK_PREFIX + user.getUserId() + ":" + mappingPath;
			redisClient.delete(sessionKey);
		}
	}

}
