package cn.com.qmth.examcloud.web.reports;

import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.web.config.SystemConfig;
import cn.com.qmth.examcloud.web.redis.RedisClient;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.ApiOperation;

/**
 * 报表采集和回调
 *
 * @author WANGWEI
 * @date 2018年7月4日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@RestController
@RequestMapping("reports")
public class ReportsController extends ControllerSupport {

	@Value("${spring.application.name}")
	String application;

	@Autowired
	RedisClient redisClient;

	/**
	 * 获取报告文件
	 *
	 * @author WANGWEI
	 * @throws IOException
	 */
	@ApiOperation(value = "获取报告文件")
	@GetMapping("getReportFile")
	public ResponseEntity<?> getReportFile(@RequestParam(required = true) String className)
			throws IOException {

		if (StringUtils.isBlank(className)) {
			throw new StatusException("580", "className is blank");
		}
		Class<?> c = null;
		try {
			c = Class.forName(className);
		} catch (ClassNotFoundException e) {
			throw new StatusException("581", "className is wrong");
		}
		if (!c.isAssignableFrom(BaseReport.class)) {
			throw new StatusException("582", "className is wrong");
		}

		String dirPath = getReportsLogRootDirectory() + "/" + className;

		File dir = new File(dirPath);

		if ((!dir.exists()) || (!dir.isDirectory())) {
			new ResponseEntity<>(HttpStatus.NO_CONTENT);
		}

		String[] fileNames = dir.list(new ReportFileFilter(c));

		if (null == fileNames || 0 == fileNames.length) {
			new ResponseEntity<>(HttpStatus.NO_CONTENT);
		}
		List<String> fileNameList = Arrays.asList(fileNames);
		Collections.sort(fileNameList);

		String fileName = fileNameList.get(0);
		String filePath = dirPath + "/" + fileName;

		File file = new File(filePath);
		HttpHeaders headers = new HttpHeaders();
		headers.setContentDispositionFormData("attachment", fileName);
		headers.setContentType(MediaType.APPLICATION_OCTET_STREAM);
		return new ResponseEntity<byte[]>(FileUtils.readFileToByteArray(file), headers,
				HttpStatus.OK);
	}

	/**
	 * 删除报告文件
	 *
	 * @author WANGWEI
	 * @throws IOException
	 */
	@GetMapping("delReportFile")
	public void delReportFile(@RequestParam(required = true) String className,
			@RequestParam(required = true) String fileName) throws IOException {

		if (StringUtils.isBlank(className)) {
			throw new StatusException("580", "className is blank");
		}
		Class<?> c = null;
		try {
			c = Class.forName(className);
		} catch (ClassNotFoundException e) {
			throw new StatusException("581", "className is wrong");
		}
		if (!c.isAssignableFrom(BaseReport.class)) {
			throw new StatusException("582", "className is wrong");
		}

		String dirPath = getReportsLogRootDirectory() + "/" + className;
		String filePath = dirPath + "/" + fileName;

		FileUtils.forceDelete(new File(filePath));
	}

	/**
	 * 获取报告根目录
	 *
	 * @author WANGWEI
	 * @return
	 */
	private String getReportsLogRootDirectory() {
		String dataDir = SystemConfig.getDataDir();
		if (!dataDir.endsWith("/")) {
			dataDir += "/";
		}
		return dataDir + "reports";
	}

}
